package com.tzld.piaoquan.ad.engine.service.predict.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.tzld.piaoquan.ad.engine.commons.enums.AppTypeEnum;
import com.tzld.piaoquan.ad.engine.commons.redis.AlgorithmRedisHelper;
import com.tzld.piaoquan.ad.engine.commons.util.DateUtils;
import com.tzld.piaoquan.ad.engine.service.predict.config.NewExpUserGroupConfig;
import com.tzld.piaoquan.ad.engine.service.predict.config.RoiModelConfig;
import com.tzld.piaoquan.ad.engine.service.predict.constant.RuleRedisKeyConst;
import com.tzld.piaoquan.ad.engine.service.predict.PredictModelService;
import com.tzld.piaoquan.ad.engine.service.predict.config.AbConfig;
import com.tzld.piaoquan.ad.engine.service.predict.container.AbTestConfigContainer;
import com.tzld.piaoquan.ad.engine.service.predict.container.ThresholdModelContainer;
import com.tzld.piaoquan.ad.engine.service.predict.container.TopOneVideoContainer;
import com.tzld.piaoquan.ad.engine.service.predict.helper.NewExpInfoHelper;
import com.tzld.piaoquan.ad.engine.service.predict.param.RoiThresholdPredictModelParam;
import com.tzld.piaoquan.ad.engine.service.predict.param.ThresholdPredictModelParam;
import com.tzld.piaoquan.ad.engine.service.predict.param.request.RoiPredictModelRequestParam;
import com.tzld.piaoquan.ad.engine.service.predict.param.request.ThresholdPredictModelRequestParam;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.*;

@Service
public class PredictModelServiceImpl implements PredictModelService {
    private final static Logger log = LoggerFactory.getLogger(PredictModelServiceImpl.class);
    @Value("${ad.own.experiment.tier:ab_test002}")
    private String adOwnExperimentTier;

    @Autowired
    private AbTestConfigContainer abTestConfigContainer;

    @Autowired
    private TopOneVideoContainer topOneVideoContainer;

    @Autowired
    AlgorithmRedisHelper redisHelper;

    @Autowired
    AbConfig abConfig;

    @Autowired
    RoiModelConfig roiModelConfig;

    @Value("${ad.predict.share0.exp.code:000}")
    private String adPredictNoShareUserExpCode;

//    @Value("${ad.predict.90d.share0.exp.code:612}")
//    private String adPredict90DNoShareUserExpCode;

    @Value("${ad.predict.immersion.exp.code:607}")
    private String adPredictImmersionExpCode;

    @Value("${ad.predict.param.testIds:0}")
    private String testIds;
    @Value("${ad.predict.without.ad.video_ids:0}")
    private String withoutAdVideoIds;

    List<Integer> appIdArr=Arrays.asList(new Integer[]{0,3,4,5,6,17,18,19,21,22});


    public Map<String, Object> adPredict(ThresholdPredictModelRequestParam requestParam) {
        String[] withoutAdVideoIdsArr=withoutAdVideoIds.split(",");
        for(String videoId:withoutAdVideoIdsArr){
            if(videoId.equals(requestParam.getVideoId()+"")){
                if(requestParam.getAppType().equals(0)
                        ||requestParam.getAppType().equals(4)
                        ||requestParam.getAppType().equals(5)
                        ||requestParam.getAppType().equals(21)
                ){
                    Map<String,Object> result=new HashMap<>();
                    result.put("ad_predict", 1);
                    result.put("no_ad_strategy", "no_ad_with_video_in_white_list");
                    return result;
                }
            }
        }

        Map<String,Object> result=new HashMap<>();
        List<Map<String,Object>> mapList=(List)requestParam.getAbExpInfo().get("ab_test002");
        Map<String,List<JSONObject>> configMap=new HashMap<>();
        //该用户所有实验合集
        Set<String> expCodes=new HashSet<>();
        for(Map<String,Object> map:mapList){
            String expCode=map.getOrDefault("abExpCode","").toString();
            expCodes.add(expCode);
            if("555".equals(expCode)){
                configMap=JSONObject.parseObject(map.get("configValue").toString(),Map.class);
            }
        }

        int hourOfDay= DateUtils.getCurrentHour();
        Boolean condition1=abTestConfigContainer.inAdTimeTest(requestParam.getAbExpInfo());
        Boolean condition2=abTestConfigContainer.containsCode(configMap,requestParam.getAbTestCode());
        if(condition1
                &&
                condition2
                &&
                abTestConfigContainer.inWithoutAdTime(configMap,requestParam.getAbTestCode(),hourOfDay)){
            result.put("ad_predict", 1);
            result.put("no_ad_strategy", "no_ad_time_with_time_plan");
            return result;
        } else if (
                (!condition1 || (condition1 && !condition2))
                        &&
                        (0 <= hourOfDay && hourOfDay < 8)) {
            result.put("ad_predict", 1);
            result.put("no_ad_strategy", "no_ad_time_with_fixed_time");
            return result;
        }
        String abtestId=null;
        String abTestConfigTag=null;
        Map<String, Object> abtestParam=null;
        String midGroup =null;
        String shareType =null;
        if(appIdArr.contains(requestParam.getAppType())){
            //先判断是否开启实验 和是否不出广告时间 而后判断默认0-8
            //不出广告时间判定
            String[] abParamArr = abConfig.getAbParams(requestParam.getAbTestCode(), requestParam.getAbExpInfo());
            if (abParamArr == null) {
                abParamArr= NewExpUserGroupConfig.newExpUserGroupMap.get(requestParam.getAppType().toString());
                if (abParamArr == null) {
                    result.put("msg", "abConfig_error");
                    return result;
                }
            }
            abtestId = abParamArr[0];
            abTestConfigTag = abParamArr[1];
            HashMap<String, Map<String, Object>> abConfigMap = abConfig.getAbConfigMap();
            abtestParam = abConfigMap.getOrDefault(abtestId + "-" + abTestConfigTag, null);
            if (abtestParam == null) {
                result.put("msg", "abConfig_error");
                return result;
            }
            // Determine the group to which mid belongs
            String groupClassKey = (String) abtestParam.get("group_class_key");
            String midGroupKeyName = RuleRedisKeyConst.KEY_NAME_PREFIX_MID_GROUP + groupClassKey + ":" + requestParam.getMid();
            midGroup = redisHelper.getString(midGroupKeyName);
//        if(expCodes.contains(adPredict90DNoShareUserExpCode)){
//            midGroup = redisHelper.getString(RuleRedisKeyConst.KEY_NAME_PREFIX_90_SHARE_TYPE+requestParam.getMid());
//        }else {
//            midGroup = redisHelper.getString(midGroupKeyName);
//        }
            //没有时为新用户或者无分享用户
            shareType =midGroup;
            if (midGroup == null) {
                midGroup = "mean_group";
                shareType="noShare";
            }
            String[] noAdMidGroupList = new String[0];
            noAdMidGroupList = ((JSONArray) abtestParam.get("no_ad_mid_group_list")).toArray(noAdMidGroupList);

            boolean inNoAdGroup = false;
            for (String group : noAdMidGroupList) {
                if (group.equals(midGroup)) {
                    inNoAdGroup = true;
                    break;
                }
            }
            //不出广告组
            if (inNoAdGroup) {
                // User is in the no-ad group, no ad should be shown
                result.put("mid_group", midGroup);
                result.put("ad_predict", 1);
                result.put("no_ad_strategy", "no_ad_mid_group_with_video");
                return result;
            }
            //top1广告不出视频
            Map<String, List<String>> noAdGroupWithVideoMapping = (Map) abtestParam.getOrDefault("no_ad_group_with_video_mapping", new HashMap<>());
            if (noAdGroupWithVideoMapping.keySet().contains(midGroup)
                    &&
                    topOneVideoContainer.inNoAdTopVideo(requestParam.getAppType().longValue(), requestParam.getVideoId())
            ) {
                result.put("mid_group", midGroup);
                result.put("ad_predict", 1);
                result.put("no_ad_strategy", "no_ad_mid_group_with_video");
                return result;
            }
        }else {
            String midGroupKeyName = RuleRedisKeyConst.KEY_NAME_PREFIX_MID_GROUP +  "class1:" + requestParam.getMid();
            midGroup = redisHelper.getString(midGroupKeyName);
            shareType =midGroup;
            if (midGroup == null) {
                midGroup = "mean_group";
                shareType="noShare";
            }
        }



        //市-中文
        requestParam.setRegion(requestParam.getRegion().replace("省", ""));
        requestParam.setCity(requestParam.getCity().replace("市", ""));
        //设置信息
        ThresholdPredictModelParam modelParam = ThresholdPredictModelParam.builder()
                .build();
        BeanUtils.copyProperties(requestParam, modelParam);
        modelParam.setDate(new Date());
        modelParam.setAbtestId(abtestId);
        modelParam.setAbTestConfigTag(abTestConfigTag);
        modelParam.setAbtestParam(abtestParam);
        modelParam.setMidGroup(midGroup);
        modelParam.setExtraParam(new HashMap<>());
        modelParam.addUserExtraFuture("shareType",shareType);
        setExtraParam(modelParam);

          if(expCodes.contains("599")|| NewExpInfoHelper.checkInNewExpGroupAndSetParamIfIn(
                  requestParam.getAppType().toString(),requestParam.getNewExpGroup(),"599",modelParam)){
//        if(randomModelExpCode!=null){
//            modelParam.addUserExtraFuture("randomModelKey",ExpCodeEnum.valueOfExpCode(randomModelExpCode).getRandomModelKey());
            result = ThresholdModelContainer.
                    getThresholdPredictModel("random")
                    .predict(modelParam);
        }else if(inExpList(expCodes,adPredictImmersionExpCode)
                  ||
                  NewExpInfoHelper.checkInNewExpGroupAndSetParamIfIn(
                          requestParam.getAppType().toString(),requestParam.getNewExpGroup(),"607",modelParam)){
//        if(randomModelExpCode!=null){
//            modelParam.addUserExtraFuture("randomModelKey",ExpCodeEnum.valueOfExpCode(randomModelExpCode).getRandomModelKey());
            result = ThresholdModelContainer.
                    getThresholdPredictModel("immersion")
                    .predict(modelParam);
        }
        else {
            Object thresholdMixFunc = abtestParam.getOrDefault("threshold_mix_func", "basic");
            result = ThresholdModelContainer.
                    getThresholdPredictModel(thresholdMixFunc.toString())
                    .predict(modelParam);
        }

        return result;
    }

    public Map<String, Object> adRecommendPredictByRoiModel(RoiPredictModelRequestParam requestParam) {

        Map<String, Object> result = new HashMap<>();
        int hourOfDay = DateUtils.getCurrentHour();
        if (hourOfDay < 8 && hourOfDay >= 0) {
            result.put("ad_predict", 1);
            result.put("no_ad_strategy", "no_ad_time");
            return result;
        }
        RoiThresholdPredictModelParam modelParam = new RoiThresholdPredictModelParam();
        BeanUtils.copyProperties(requestParam, modelParam);
        modelParam.setEcpm(requestParam.getAds().get(0).getEcpm());
        modelParam.setAdId(requestParam.getAds().get(0).getAdId());
        modelParam.setAdType(requestParam.getAds().get(0).getAdType());

        String groupClassKey = (String) ((Map<String, Object>) roiModelConfig.get(AppTypeEnum.valueOf(modelParam.getAppType()).name())).get("group_class_key");
        String midGroupKeyName = RuleRedisKeyConst.KEY_NAME_PREFIX_MID_GROUP + groupClassKey + ":" + modelParam.getMid();
        String midGroup = redisHelper.get(midGroupKeyName);
        if (midGroup == null) {
            midGroup = "mean_group";
        }
        modelParam.setMidGroup(midGroup);

        return ThresholdModelContainer.
                getThresholdPredictModel("roi")
                .predict(modelParam);
    }

    public void setExtraParam(ThresholdPredictModelParam modelParam){
        String[] ids=testIds.split(",");
        List<String> idList=Arrays.asList(ids);
        List<Map<String,Object>> mapList=(List)modelParam.getAbExpInfo().get("ab_test002");
        Collections.sort(mapList,new Comparator<Map<String, Object>>() {
            @Override
            public int compare(Map<String, Object> map1, Map<String, Object> map2) {
                int abExpCode1 =Integer.parseInt(map1.get("abExpCode").toString()) ;
                int abExpCode2 =Integer.parseInt(map2.get("abExpCode").toString());
                return Integer.compare(abExpCode1, abExpCode2);
            }
        });
        Map<String,Object> configMap;
        for(Map<String,Object> map:mapList){
            if(idList.contains(map.getOrDefault("abExpCode",""))){
                configMap=JSONObject.parseObject(map.get("configValue").toString(),new TypeReference<Map<String,Object>>(){});
                for(Map.Entry<String,Object> entry:configMap.entrySet()){
                    modelParam.getExtraParam().put(entry.getKey(),entry.getValue());
                }
            }
        }


    }


    boolean inExpList(Set<String> set,String expCodes){
        String[] expArr=expCodes.split(",");
        for(String str:expArr){
            if(set.contains(str)){
                return true;
            }
        }
        return false;
    }
}
